import subprocess

from ajenti.api import *
from api import PackageInfo, PackageManager


@plugin
class DebianPackageManager (PackageManager):
    platforms = ['debian']

    def refresh(self):
        out_u = subprocess.check_output(['apt-show-versions', '-u'])
        out_a = subprocess.check_output(['dpkg', '-l'])
        self.all = self._parse_dpkg(out_a)
        self.all_dict = dict((x.name, x) for x in self.all)
        self.upgradeable = self._parse_asv(out_u)

    def search(self, query):
        out_s = subprocess.check_output(['apt-show-versions', '-a', '-R', query])
        r = []
        found = {}
        for l in out_s.split('\n'):
            s = l.split()
            if len(s) < 4:
                continue

            p = PackageInfo()
            p.name = s[0]
            p.state = 'i' if p.name in self.all_dict else 'r'
            p.version = s[1]

            if not p.name in found or found[p.name] < p.version:
                r.append(p)
                found[p.name] = p.version
        return r

    def get_lists(self):
        self.context.launch('terminal', command='apt-get update')

    def do(self, actions, callback=lambda: 0):
        cmd = 'apt-get install '
        for a in actions:
            cmd += a.name + {'r': '-', 'i': '+'}[a.action] + ' '
        self.context.launch('terminal', command=cmd, callback=callback)

    def _parse_asv(self, d):
        r = []
        for l in d.split('\n'):
            s = l.split('/')
            if len(s) == 0 or not s[0]:
                continue

            p = PackageInfo()
            p.name = s[0]
            p.version = s[1].split(' ')[-1]
            r.append(p)
        return r

    def _parse_apt(self, d):
        r = []
        for l in d.split('\n'):
            s = filter(None, l.split(' '))
            if len(s) == 0:
                continue

            p = PackageInfo()
            if s[0] == 'Inst':
                p.action = 'i'
            elif s[0] in ['Remv', 'Purg']:
                p.action = 'r'
            else:
                continue
            p.name = s[1]
            p.version = s[2].strip('[]')
            r.append(p)
        return r

    def _parse_dpkg(self, d):
        r = []
        for l in d.split('\n'):
            s = filter(None, l.split(' '))
            if len(s) == 0:
                continue

            p = PackageInfo()
            if s[0][0] == 'i':
                p.state = 'i'
            else:
                continue

            p.name = s[1]
            p.version = s[2]
            p.description = ' '.join(s[3:])
            r.append(p)
        return r



import subprocess
import re
import os

from ajenti.api import plugin
from ajenti.api.sensors import Sensor
from ajenti.plugins.dashboard.api import ConfigurableWidget


@plugin
class SMARTSensor (Sensor):
    id = 'smart'
    timeout = 5

    def get_variants(self):
        r = []
        for s in os.listdir('/dev'):
            if re.match('sd.$|hd.$|scd.$|fd.$|ad.+$', s):
                r.append(s)
        return sorted(r)

    def measure(self, path):
        """
        -1 = No SMART
        0  = DISK FAILING
        1  = PRE-FAIL
        2  = Unknown error
        3  = Errors in log
        4  = DISK OK
        """
        if not path:
            return -1
        r = subprocess.call(['smartctl', '-H', '/dev/' + path])
        if r & 2:
            return -1
        if r & 8:
            return 0
        if r & 16:
            return 1
        if r & 64:
            return 3
        if r == 0:
            return 4
        return 2


@plugin
class SMARTWidget (ConfigurableWidget):
    name = 'S.M.A.R.T.'
    icon = 'hdd'

    def on_prepare(self):
        self.sensor = Sensor.find('smart')
        self.append(self.ui.inflate('smartctl:widget'))

    def on_start(self):
        self.find('device').text = self.config['device']
        v = self.sensor.value(self.config['device'])
        v = {
            -1: _('No data'),
            0: _('FAILING'),
            1: _('PRE-FAIL'),
            2: _('Unknown error'),
            3: _('Errors in log'),
            4: 'OK'
        }[v]
        self.find('value').text = v

    def create_config(self):
        return {'device': ''}

    def on_config_start(self):
        device_list = self.dialog.find('device')
        lst = self.sensor.get_variants()
        device_list.labels = lst
        device_list.values = lst
        device_list.value = self.config['device']

    def on_config_save(self):
        self.config['device'] = self.dialog.find('device').value



import time
import subprocess

from ajenti.api import *
from ajenti.ui import *


@plugin
class LinuxIfconfig (object):
    platforms = ['debian', 'centos']

    def detect_dev_class(self, iface):
        if iface.name[:-1] in ['ppp', 'wvdial']:
            return 'ppp'
        if iface.name[:-1] in ['wlan', 'ra', 'wifi', 'ath']:
            return 'wireless'
        if iface.name[:-1] == 'br':
            return 'bridge'
        if iface.name[:-1] == 'tun':
            return 'tunnel'
        if iface.name == 'lo':
            return 'loopback'
        if iface.name[:-1] == 'eth':
            return 'ethernet'
        return 'ethernet'

    def detect_iface_bits(self, iface):
        r = ['linux-basic']
        cls = self.detect_dev_class(iface)
        if iface.type == 'inet' and iface.addressing == 'static':
            r.append('linux-ipv4')
        if iface.type == 'inet6' and iface.addressing == 'static':
            r.append('linux-ipv6')
        if iface.addressing == 'dhcp':
            r.append('linux-dhcp')
        if cls == 'ppp':
            r.append('linux-ppp')
        if cls == 'wireless':
            r.append('linux-wlan')
        if cls == 'bridge':
            r.append('linux-bridge')
        if cls == 'tunnel':
            r.append('linux-tunnel')

        r.append('linux-ifupdown')
        return r

    def up(self, iface):
        subprocess.call(['ifup', iface.name])
        time.sleep(1)

    def down(self, iface):
        subprocess.call(['ifdown', iface.name])
        time.sleep(1)



from ajenti.api import *
from ajenti.plugins.main.api import SectionPlugin
from ajenti.ui import on
from ajenti.ui.binder import Binder

from backend import *


@plugin
class LTFSTapegroupsPlugin (SectionPlugin):
    def init(self):
        self.title = 'Tape Groups'
        self.icon = 'hdd'
        self.category = 'LTFS'

        self.append(self.ui.inflate('ltfs:tapegroups-main'))
        self.tgbackend = TapeGroupBackend()

        self.tgbinder = Binder(self.tgbackend, self.find('tapegroups'))
        self.find('groups').new_item = lambda c: TapeGroup()
        self.find('tapes').new_item = lambda c: Tape()

        def post_tape_bind(object, collection, item, ui):
            def move():
                self.tgbinder.update()
                for g in self.tgbackend.groups:
                    if item in g.tapes:
                        g.tapes.remove(item)
                    if g.id == ui.find('destination').value:
                        g.tapes.append(item)
                self.tgbinder.populate()
            ui.find('move').on('click', move)
            ui.find('destination').labels = [g.name for g in self.tgbackend.groups]
            ui.find('destination').values = [g.id for g in self.tgbackend.groups]

        self.find('tapes').post_item_bind = post_tape_bind

    def refresh(self):
        self.tgbackend.reload()
        self.tgbinder.reset().autodiscover().populate()

    def on_page_load(self):
        self.refresh()

    @on('refresh', 'click')
    def on_refresh(self):
        self.refresh()

    @on('save', 'click')
    def on_save(self):
        self.tgbinder.update()
        self.tgbackend.save()
        self.refresh()



import subprocess


class SambaUser (object):
    def __init__(self):
        self.username = None
        self.sid = None


class SambaUsers (object):
    def load(self):
        self.users = []
        for un in [s.split(':')[0] for s in subprocess.check_output(['pdbedit', '-L', '-d0']).split('\n')]:
            if un and not ' ' in un and not un.startswith('WARNING'):
                lines = subprocess.check_output(['pdbedit', '-Lv', '-d0', '-u', un]).split('\n')
                fields = {}
                for l in lines:
                    if l and ':' in l:
                        l = l.split(':', 1)
                        fields[l[0]] = l[1].strip()
                u = SambaUser()
                u.username = un
                u.sid = fields['User SID']
                self.users.append(u)

    def create(self, un):
        p = subprocess.Popen(['pdbedit', '-at', '-u', un])
        p.communicate('\n\n\n')

    def delete(self, un):
        subprocess.call(['pdbedit', '-x', '-u', un])

    def set_password(self, un, p):
        p = subprocess.Popen(['pdbedit', '-rt', '-u', un])
        p.communicate('%s\n%s\n' % (p, p))
        return p.returncode == 0



import os
import subprocess

from ajenti.api import *

from api import *
from nctp_linux import *


optionmap = {
    'IPADDR': 'address',
    'NETMASK': 'netmask',
    'GATEWAY': 'gateway',
    'BROADCAST': 'broadcast',
    'MACADDR': 'hwaddr',
}


@plugin
class CentosNetworkConfig (LinuxIfconfig, INetworkConfig):
    platforms = ['centos']

    interfaces = None

    classes = {
        'none': ('inet', 'static'),
        'dhcp': ('inet', 'dhcp'),
        'loopback': ('inet', 'loopback')
    }

    def __init__(self):
        self.rescan()

    def rescan(self):
        self.interfaces = {}

        for ifcf in os.listdir('/etc/sysconfig/network-scripts/'):
            if ifcf.startswith('ifcfg-'):
                ifcn = ifcf[6:]
                with open('/etc/sysconfig/network-scripts/' + ifcf, 'r') as f:
                    ss = f.read().splitlines()
                    d = {}
                    for s in ss:
                        try:
                            k = s.split('=')[0].strip('\t "\'')
                            v = s.split('=')[1].strip('\t "\'')
                            d[k] = v
                        except:
                            pass

                    m = 'loopback'
                    c = 'inet'
                    try:
                        if 'BOOTPROTO' in d:
                            m = d['BOOTPROTO']
                        c, m = self.classes[m]
                    except:
                        pass

                    e = NetworkInterface()
                    e.name = ifcn
                    self.interfaces[ifcn] = e
                    self.interfaces[ifcn].auto = 'yes' in d.get('ONBOOT', 'no')
                    e.type = c
                    e.addressing = m
                    for k in d:
                        if k == 'BOOTPROTO':
                            pass
                        elif k in optionmap:
                            e.params[optionmap[k]] = d[k]
                        else:
                            e.params[k] = d[k]

                    e.devclass = self.detect_dev_class(e)
                    try:
                        e.up = 'UP' in subprocess.check_output(['ifconfig', e.name])
                    except:
                        e.up = False
                    e.bit_classes = self.detect_iface_bits(e)

    def save(self):
        for i in self.interfaces:
            with open('/etc/sysconfig/network-scripts/ifcfg-' + i, 'w') as f:
                self.save_iface(self.interfaces[i], f)
        return

    def save_iface(self, iface, f):
        iface.params['ONBOOT'] = 'yes' if iface.auto else 'no'
        for x in self.classes:
            if self.classes[x] == (iface.type, iface.addressing):
                f.write('BOOTPROTO="' + x + '"\n')

        for x in iface.params:
            fnd = False
            for k in optionmap:
                if optionmap[k] == x:
                    f.write(k + '="' + iface.params[x] + '"\n')
                    fnd = True
            if not fnd:
                f.write(x + '="' + iface.params[x] + '"\n')



import os
import fcntl
import pty
import gevent

import pyte


class Terminal (object):
    def __init__(self, command=None, autoclose=False, **kwargs):
        self.protocol = None
        self.width = 160
        self.height = 35
        self.autoclose = autoclose

        env = {}
        env.update(os.environ)
        env['TERM'] = 'linux'
        env['COLUMNS'] = str(self.width)
        env['LINES'] = str(self.height)
        env['LC_ALL'] = 'en_US.UTF8'

        command = ['sh', '-c', command or 'sh']

        pid, master = pty.fork()
        if pid == 0:
            os.execvpe('sh', command, env)

        self.screen = pyte.DiffScreen(self.width, self.height)
        self.protocol = PTYProtocol(pid, master, self.screen, **kwargs)

    def restart(self):
        if self.protocol is not None:
            self.protocol.kill()
        self.start()

    def dead(self):
        return self.protocol is None or self.protocol.dead

    def write(self, data):
        self.protocol.write(data)

    def kill(self):
        self.protocol.kill()


class PTYProtocol():
    def __init__(self, pid, master, term, callback=None):
        self.data = ''
        self.pid = pid
        self.master = master
        self.dead = False
        self.callback = callback

        fd = self.master
        fl = fcntl.fcntl(fd, fcntl.F_GETFL)
        fcntl.fcntl(fd, fcntl.F_SETFL, fl | os.O_NONBLOCK)

        self.mstream = os.fdopen(self.master, 'r+')
        gevent.sleep(0.5)
        self.term = term
        self.stream = pyte.Stream()
        self.stream.attach(self.term)
        self.data = ''

    def read(self):
        for i in range(0, 45):
            try:
                d = self.mstream.read()
                self.data += d
                if len(self.data) > 0:
                    u = unicode(str(self.data))
                    self.stream.feed(u)
                    self.data = ''
                break
            except IOError:
                pass
            except UnicodeDecodeError:
                pass
            gevent.sleep(0.33)

        self._check()
        return self.format()

    def _check(self):
        pid, status = os.waitpid(self.pid, os.WNOHANG)
        if pid:
            self.on_died(code=status)

    def on_died(self, code=0):
        if self.dead:
            return
        self.dead = True
        if code:
            self.stream.feed('\n\n * ' + unicode(_('Process has exited with status %i') % code))
        else:
            self.stream.feed('\n\n * ' + unicode(_('Process has exited successfully')))
        if self.callback:
            try:
                self.callback(exitcode=code)
            except TypeError:
                self.callback()

    def history(self):
        return self.format(full=True)

    def format(self, full=False):
        l = {}
        self.term.dirty.add(self.term.cursor.y)
        for k in self.term.dirty:
            l[k] = self.term[k]
        self.term.dirty.clear()
        r = {
            'lines': self.term if full else l,
            'cx': self.term.cursor.x,
            'cy': self.term.cursor.y,
            'cursor': not self.term.cursor.hidden,
        }
        return r

    def write(self, data):
        self.mstream.write(data)
        self.mstream.flush()

    def kill(self):
        os.kill(self.pid, 9)



from ajenti.api import plugin
from ajenti.plugins.dashboard.api import ConfigurableWidget


@plugin
class TextWidget (ConfigurableWidget):
    name = _('Text')
    icon = 'font'

    def on_prepare(self):
        self.append(self.ui.inflate('dashboard:text'))

    def on_start(self):
        self.find('text').text = self.config['text']

    def create_config(self):
        return {'text': ''}

    def on_config_start(self):
        pass

    def on_config_save(self):
        self.config['text'] = self.dialog.find('text').value



import logging
import threading
import uuid

from ajenti.api import *


class TaskResult (object):
    SUCCESS = 0
    ABORTED = 1
    ERROR = 2
    CRASH = 3

    def __init__(self):
        self.result = TaskResult.SUCCESS
        self.output = ''
        self.name = None


class TaskError (Exception):
    pass


@interface
class Task (object):
    """
    Base class for custom tasks
    
    :param name: display name
    :param ui: full layout name for parameter editor, will be bound to parameter dictionary (so begin it with <bind:dict bind="params">)
    :param hidden: if True, task won't be available for manual creation
    """

    name = '---'
    ui = None
    hidden = False

    def __init__(self, **kwargs):
        self.params = kwargs

    def init(self):
        self._progress = 0
        self._progress_max = 1
        self.running = False
        self.complete = False
        self.pending = False
        self.aborted = False
        self.parallel = False
        self.message = ''
        self.result = TaskResult()
        self.callback = lambda x: None

    def start(self):
        self.thread = threading.Thread(target=self._run)
        self.thread.start()

    def _run(self):
        logging.info('Starting task %s' % self.__class__.__name__)
        self.running = True
        self.result.name = self.name
        try:
            self.run(**self.params)
            self.result.result = TaskResult.SUCCESS
        except TaskError, e:
            self.result.result = TaskResult.ERROR
            self.result.output = e.message
        except Exception, e:
            self.result.result = TaskResult.CRASH
            self.result.output = str(e)
            logging.exception(str(e))
        self.running = False
        self.complete = True 
        logging.info('Task %s complete (%s)' % (self.__class__.__name__, 'aborted' if self.aborted else 'success'))
        self.callback(self)

    def run(**kwargs):
        """
        Override with your task actions here.
        Raise :class:`TaskError` in case of emergency.
        Check `aborted` often and return if it's True

        :param **kwargs: saved task parameters
        """

    def abort(self):
        if not self.running:
            return
        self.aborted = True
        self.result.result = TaskResult.ABORTED
        self.thread.join()
        
    def set_progress(self, current, max):
        self._progress, self._progress_max = current, max

    def get_progress(self):
        return self._progress, self._progress_max


class TaskDefinition (object):
    def __init__(self, j={}, task_class=None):
        self.name = j.get('name', 'unnamed')
        self.parallel = j.get('parallel', False)
        self.task_class = j.get('task_class', task_class)
        self.params = j.get('params', self.get_class().default_params.copy() if self.get_class() else {})
        self.id = j.get('id', str(uuid.uuid4()))

    def get_class(self):
        for task in Task.get_classes():
            if task.classname == self.task_class:
                return task

    def save(self):
        return {
            'name': self.name,
            'task_class': self.task_class,
            'params': self.params,
            'id': self.id,
        }


class JobDefinition (object):
    def __init__(self, j={}):
        self.name = j.get('name', 'unnamed')
        self.task_id = j.get('task_id', None)
        self.id = j.get('id', str(uuid.uuid4()))
        self.schedule_special = j.get('schedule_special', None)
        self.schedule_minute = j.get('schedule_minute', '0')
        for _ in ['hour', 'day_of_month', 'month', 'day_of_week']:
            setattr(self, 'schedule_' + _, j.get('schedule_' + _, '*'))

    def save(self):
        return {
            'name': self.name,
            'task_id': self.task_id,
            'id': self.id,
            'schedule_special': self.schedule_special,
            'schedule_minute': self.schedule_minute,
            'schedule_hour': self.schedule_hour,
            'schedule_day_of_month': self.schedule_day_of_month,
            'schedule_month': self.schedule_month,
            'schedule_day_of_week': self.schedule_day_of_week,
        }



# -*- coding: utf-8 -*-


import hashlib
import sqlalchemy
import sqlalchemy.orm
from sqlalchemy.ext.declarative import declarative_base
import sqlamp
import os
import time
import locale

#print locale.getpreferredencoding()
#print sys.stdout.encoding


class TapeLibraryHandler():

    def __init__(self):
        self.options = {}

        # after a depth of 'iMaxDepth_sqlamp' in hierarchy just dump a "tree"
        # call as text (instead of inserting each file/folder as a node to
        # the database)
        # raising this number will make the traversal a lot slower
        # leaving this in as this will build a nodegraph in the database
        # which might come in handy later
        # (what really gets easy is moving a directory or something like that)
        # (also finding the parent directories is a little easier with this)
        # but for now, speed is most important...
        # also note that the database field for the text that holds the "tree"
        # output cannot be larger than 2^32 characters, so if we have gigantic
        # trees we might run into DB-errors (in this case raising this number
        # will make individual dumps smaller thus prohibit a "too-much-data" error)
        self.options["iMaxDepth_sqlamp"] = 1

        # this will control the display depth of the call to "tree"
        # leaving this at zero will not limit the tree at all
        # a value of 5 means cutting of the "tree" at 5 hierarchy levels
        # WARNING! this will result in data not being present in the database
        # (everything below the hierarchy level wont be in there)
        # I've only put this in as a last means (speed vs. quality)
        self.options["iMaxDepth_TreeDump"] = 0

        # if we want to store/find Sonderzeichen, this is also a little costly
        # but its not so bad, better leave it on...
        # True traversing took 1860.133 ms, finding took 285.214 ms
        # (propably "tree" has to do more work)
        # False traversing took 2122.753 ms, finding took 193.997 ms
        self.options["bRespectUnicode"] = True

        # if bTreeDumpFullPaths is enabled it will store the full path
        # for each file, if it is disabled, it will only store the filename
        # plus those liny things indicating hierarchy
        #
        #    |-- LTFS_Client
        #    |   |-- 01_From
        #    |   |   `-- 2013_01_29
        #    |   |       |-- ajenti
        #    |   |       |   |-- ajenti-0.6.2-1.noarch.rpm
        #
        # storing only the filenames has pros and cons:
        # pro: less memory consumption (about 35% the size of fullpaths)
        # pro: less time spent converting unicode (very minor)
        # pro: when searching this will only return distinct hits
        #      (otherwise we'll get hits for every file in a _directory_
        #      matching the searchterm as well)
        # con: the liny hierarchy indicators seem to be different
        #      for different shells (unicode vs. ascii)
        # con: a lot! more time spent to find which the actual parent directory
        #      really is (need to parse lines above)
        #
        # for now speed is most important so leave this at 'True'
        self.options["bTreeDumpFullPaths"] = True

        # find the exact entry in a dump
        # to be precise, this will reconstruct the full path for each entry
        # this can be quite costly because we dont store the full path in the
        # database but only the form of the "tree" call
        # (filename plus indicators of hierarchy)
        # also see option "bTreeDumpFullPaths" above
        # this in turn (we could also store the fullpath in the dump)
        # is a conscience choice [this way we dont get extra search hits
        # for a file's folder plus it takes less space in the database]
        # note: to see the exact output you need a iLogLevel of at least 2
        self.options["bFindExactly"] = True

        # exclude hidden directories (while writing nodes to the databse)
        # this has no effect on the "tree" call
        self.options["bExludeHidden"] = True

        # be talkative for debugging
        # the higher this value, the more it will spit (print) out
        self.options["iLogLevel"] = 0

        # add all nodes at once (this is like 15% faster)
        self.options["bAddAll"] = True
        self.lNodes = []

    def lic_InitDB(self, sUser="syslink",
                        sPass="syslink",
                        sDBName="syslink_tapelibrary",
                        sDBType="postgresql"):

        # for SQL authentication
        # for mysql we only need a user with enough rights ("DBManager")
        # for postgresql we need to make a database by hand in advance
        sSQLUser = sUser
        sSQLPass = sPass
        sSQLDBname = sDBName

        # choice of
        # "mysql" # cannot perform test because of broken pipe bug
        # "sqlite"  # 5224.302 ms | 1624.892 ms on "/" 74MB
        # "sqlitememory"  # 3774.578 ms | 1494.738 ms on "/"
        # "postgresql"  # 6607.466 ms | 1766.244 ms on "/" ~67MB
        sDatabaseType = sDBType

        if sDatabaseType == "sqlitememory":
            engine = sqlalchemy.create_engine('sqlite:///:memory:',
                                        echo=False,
                                        convert_unicode=True,
                                        encoding='utf-8')

        elif sDatabaseType == "mysql":
            # hm, seems to be an issue with mysql and longtext here
            # on lager dumps I am getting errors of "broken pipe"
            # maybe we can stick with sqlite?
            engine = sqlalchemy.create_engine('mysql+mysqlconnector://%s:%s@localhost'
                                        % (sSQLUser, sSQLPass),
                                        echo=False,
                                        convert_unicode=False,
                                        encoding='utf-8')
            try:
                engine.execute("CREATE DATABASE IF NOT EXISTS %s" % sSQLDBname)
            except:
                pass
            engine.execute("USE %s" % sSQLDBname)  # select new db

        elif sDatabaseType == "sqlite":
            sDatabaseStorage = "/lichtwerk/01_projekte/0065_LTFS_SyslinkLTFS/LTFS_Produktion/01_ScriptsPlugins/database_storage/syslink.db"
            engine = sqlalchemy.create_engine('sqlite:///%s' % sDatabaseStorage,
                                        echo=False,
                                        convert_unicode=True,
                                        encoding='utf-8')

        elif sDatabaseType == "postgresql":
            # note: syslink has sqlalchemy 0.5.5 installed (different syntax)
            #engine = sqlalchemy.create_engine('postgres://%s:%s@localhost/%s' \
            engine = sqlalchemy.create_engine('postgresql+psycopg2://%s:%s@localhost/%s' \
                                        % (sSQLUser, sSQLPass, sSQLDBname),
                                        echo=False,
                                        convert_unicode=True,
                                        encoding='utf-8')

        metadata = sqlalchemy.MetaData(engine)

        BaseNode = declarative_base(metadata=metadata,
                                    metaclass=sqlamp.DeclarativeMeta)

        global DBContentNode
        class DBContentNode(BaseNode):
            __tablename__ = 'node'
            __mp_manager__ = 'mp'
            id = sqlalchemy.Column(sqlalchemy.Integer, primary_key=True)
            parent_id = sqlalchemy.Column(sqlalchemy.ForeignKey('node.id',
                                            ondelete='CASCADE'))
            parent = sqlalchemy.orm.relation("DBContentNode", remote_side=[id])
            checksum = sqlalchemy.Column(sqlalchemy.UnicodeText())

            if sDatabaseType.find("sqlite") != -1:
                name = sqlalchemy.Column(sqlalchemy.Unicode())
                fullpath = sqlalchemy.Column(sqlalchemy.Unicode())
                treedump = sqlalchemy.Column(sqlalchemy.UnicodeText())
            if sDatabaseType.find("mysql") != -1:
                name = sqlalchemy.Column(sqlalchemy.Unicode(512))
                fullpath = sqlalchemy.Column(sqlalchemy.Unicode(512))
                treedump = sqlalchemy.Column(sqlalchemy.dialects.mysql.LONGTEXT(unicode=True))
            if sDatabaseType.find("postgresql") != -1:
                name = sqlalchemy.Column(sqlalchemy.Unicode())
                fullpath = sqlalchemy.Column(sqlalchemy.Unicode())
                treedump = sqlalchemy.Column(sqlalchemy.UnicodeText())

            def __init__(self, name='', fullpath='', parent=None, treedump='', checksum=''):
                self.name = name
                self.fullpath = fullpath
                self.parent = parent
                self.treedump
                self.checksum = checksum

            def __repr__(self):
                return '<DBContentNode %r>' % self.fullpath

        global DBContentTape
        class DBContentTape(BaseNode):
            __tablename__ = 'tape'
            id = sqlalchemy.Column(sqlalchemy.Integer, primary_key=True)
            rootnode_id = sqlalchemy.Column(sqlalchemy.ForeignKey('node.id'))
            name = sqlalchemy.Column(sqlalchemy.Unicode(255))
            custom_a = sqlalchemy.Column(sqlalchemy.UnicodeText())
            custom_b = sqlalchemy.Column(sqlalchemy.UnicodeText())
            custom_c = sqlalchemy.Column(sqlalchemy.UnicodeText())
            custom_d = sqlalchemy.Column(sqlalchemy.UnicodeText())

            def __init__(self, name='', rootnode=None,
                        custom_a="",
                        custom_b="",
                        custom_c="",
                        custom_d="",):
                self.name = name
                if rootnode:
                    self.rootnode_id = rootnode.id
                self.custom_a = custom_a
                self.custom_b = custom_b
                self.custom_c = custom_c
                self.custom_d = custom_d

            def __repr__(self):
                return '<DBContentTape %r>' % self.name

        self.DBContentNode = DBContentNode
        self.DBContentTape = DBContentTape

        BaseNode.metadata.create_all(engine)
        self.session = sqlalchemy.orm.sessionmaker(engine)()

    def log(self, sString, iLevel=0):
        if iLevel <= self.options["iLogLevel"]:
            print sString

    def string_UnicodeConvert(self, sString):
        '''
        we shoud be operating on unicode everywhere (Sonderzeichen etc.)
        so to ensure we can compare strings correctly we'll have to run them
        through this function
        also: output of "tree" e.g might have different encoding...
        '''
        try:
            sString = sString.decode(locale.getpreferredencoding())
        except Exception, e:
            self.log(e, 0)

        return sString

    def lic_ConstructFullPath(self, oNode):
        '''
        note: not used anymore
        this was used in case of storing all nodes in the database via sqlamp
        we used to only store the directory name and not the full path
        so we could reconstruct the fullpath here by traversing sqlamp "ancestors"
        '''
        #print "constructing full path for %s" % oNode.name

        lAnchestors = oNode.mp.query_ancestors(and_self=True).all()
        if len(lAnchestors) == 1:
            # we have the root node
            return oNode.name
        else:
            return os.sep.join([oAnchestor.name for oAnchestor in lAnchestors])

    def lic_CheckTreeDumpStyle(self, lLines):
        '''
        on my systems, a "tree" call can have differently styled output
        sometimes it is
        "├"
        "└"
        sometimes it is
        "|--"
        "`--"
        '''
        sTokenDirect_unicode = u"├"
        sTokenDirect_ascii = "|--"

        sStyle = "unknown"
        for sLine in lLines:
            if sLine.find(sTokenDirect_unicode) != -1:
                sStyle = "unicode"
                return sStyle

            elif sLine.find(sTokenDirect_ascii) != -1:
                sStyle = "ascii"
                return sStyle

        return sStyle

    def lic_SplitTreeLine(self, sLine, sDumpStyle):
        '''
        if we dont dump the fullpath for each file (but only the filename
        plus those liny indicators of hierarchy) this function will split those
        lines into the relevant parts (sHierarchy, sFile, iLevels)
        note: this doesnt seem to use the same tokens in all shells
        sometimes this uses
        "├"
        "└"
        sometimes it uses
        "|--"
        "`--"
        '''

        if sDumpStyle == "unknown":
            return False

        sTokenDirect1_unicode = u"├"
        sTokenDirect2_unicode = u"└"
        sTokenDirect1_ascii = "|--"
        sTokenDirect2_ascii = "`--"

        iFound = sLine.find(eval('sTokenDirect1_%s' % sDumpStyle))
        if iFound == -1:
            iFound = sLine.find(eval('sTokenDirect2_%s' % sDumpStyle))
            if iFound == -1:
                self.log("ignoring line %s" % sLine, 3)

                return False

        sHierarchy = sLine[:iFound]
        sFile = sLine[iFound + 4:]
        iLevels = len(sHierarchy) / 4

        return (sHierarchy, sFile, iLevels)

    def lic_ConstructFullPathTree(self, iLineIndexOrig,
                                    iLevelsOrig,
                                    lAllLines,
                                    sResult,
                                    sDumpStyle):
        '''
        this will dig through a treedump of the form which doesnt have full paths
        but only the filenames plus those liny indicators of hierarchy
        in this case we dont know beforehand which the actual parent folder is for an entry like this

        |-- LTFS_Client
        |   |-- 01_From
        |   |   `-- 2013_01_29
        |   |       |-- ajenti
        |   |       |   |-- ajenti-0.6.2-1.noarch.rpm

        so we have to traverse upwards line by line to find a parent...

        note: the result of a "tree" call doesnt seem to use the same tokens in all shells:
        sometimes this uses
        "├"
        "└"
        sometimes it uses
        "|--"
        "`--"
        this aint good for parsing [but is respected in lic_SplitTreeLine()]

        one strategy to speed this up would be to cache this in a dictionary
        where we store already found parents [with linenumbers etc.]
        so at least we dont have to make the lookup multiple times for same entries...
        '''
        self.log("looking for parent of %s" % lAllLines[iLineIndexOrig], 2)

        bFoundParent = False
        iLineIndexParentCheck = iLineIndexOrig - 1  # we want to look upwards
        lRange = range(len(lAllLines) - 1)

        while not bFoundParent:
            if iLineIndexParentCheck in lRange:
                t = self.lic_SplitTreeLine(lAllLines[iLineIndexParentCheck],
                                            sDumpStyle)
                if t:
                    (sHierarchyCheck, sFileCheck, iLevelsCheck) = t
                    if iLevelsCheck < iLevelsOrig:
                        sResult = "%s/%s" % (sFileCheck, sResult)
                        bFoundParent = True

                        self.log("parent now %s" % sResult, 3)

                        # recursive here we go
                        sResult = self.lic_ConstructFullPathTree(
                                                        iLineIndexParentCheck,
                                                        iLevelsCheck,
                                                        lAllLines,
                                                        sResult,
                                                        sDumpStyle)
                else:
                    # we came to the top
                    bFoundParent = True

                iLineIndexParentCheck -= 1

        return sResult

    def lic_TraverseDirectory(self, oParent, iDepth, checksums={}):
        '''
        this is getting the directory contents (file- and directory names)
        recursively. please read the comments for options "iMaxDepth_sqlamp",
        "iMaxDepth_TreeDump", "bRespectUnicode" and "iMaxDepth_TreeDump"
        to see how this works internally

        TODO: broken pipe problems with mysql and large texts (> 1.000.000)
        SOLVED: by using postgresql instead :)
        '''

        # insert a simple "tree" call dump instead
        # of continueing to insert real nodes into the database
        if iDepth >= self.options["iMaxDepth_sqlamp"]:

            sOptions = ""
            if self.options["bRespectUnicode"]:
                # [is important for special character encoding...]
                sOptions = "%s -N" % sOptions
            if self.options["bTreeDumpFullPaths"]:
                # [will print fullpath prefix for each file]
                sOptions = "%s -f " % sOptions
            if self.options["iMaxDepth_TreeDump"] > 0:
                sOptions = "%s -L %s" % (sOptions,
                                        self.options["iMaxDepth_TreeDump"])

            sDump = os.popen("tree %s %s" % (sOptions, oParent.fullpath)).read()

            # this is a little costly, but have to do for Sonderzeichen...
            if self.options["bRespectUnicode"]:
                sDump = self.string_UnicodeConvert(sDump)

            oParent.treedump = sDump

            if iDepth == 0:
                # special case for root node (this has been comitted by hand)
                self.session.add(oParent)
                self.session.commit()

            return

        # if "iMaxDepth_sqlamp" is not reached > continue inserting real nodes
        # of files/folders into the database
        sParentFullPath = oParent.fullpath

        try:
            lEntries = os.listdir(sParentFullPath)
        except:
            lEntries = []
            self.log("cannot determine dirlist for %s" % sParentFullPath, 1)

        for s in lEntries:

            if s[0] == "." and self.options["bExludeHidden"]:
                continue

            # this is a bit (not much) costly, but have to do for special chars
            if self.options["bRespectUnicode"]:
                s = self.string_UnicodeConvert(s)

            # add the node
            sFullPath = os.sep.join([sParentFullPath, s])

            checksum = ''
            if len(sFullPath.split('/')) > 3:
                sPartPath = '/' + sFullPath.split('/', 3)[3]
                if sPartPath in checksums:
                    checksum = checksums[sPartPath]

            oNewNode = DBContentNode(
                name=s,
                fullpath=sFullPath,
                parent=oParent,
                checksum=checksum
            )
            self.lic_AddNode(oNewNode)

            # trverse recursevly if we find a directory
            if os.path.isdir(sFullPath):
                #print "found directory %s" % sFullPath
                self.lic_TraverseDirectory(oNewNode, iDepth + 1, checksums)

    def lic_AddNode(self, oNewNode):
        if self.options["bAddAll"]:
            self.lNodes.append(oNewNode)
        else:
            self.session.add(oNewNode)
            self.session.commit()

    def lic_CreateTape(self, sRootPath, sTapeName,
                        custom_a="",
                        custom_b="",
                        custom_c="",
                        custom_d="",
                        checksumFile=None):

        if not os.path.exists(sRootPath):
            print "rootpath doesnt exist"
            self.log("rootpath doesnt exist", 0)
            return

        '''
        # === see if everything is in place
        if self.options["sDatabaseType"] == "mysql":
            # mysql running?
            sMySQL = os.popen("ps aux").read()
            if sMySQL.find("mysqld") != -1:
                self.log("OK    mysqld seems to be running", 0)
            else:
                self.log("ERROR mysqld seems to be down", 0)
        '''

        if self.options["iLogLevel"] > 0:
            # lets do some profiling
            t1 = time.time()

        # construct a root node for this tape
        # add and commit to session by hand (because we need inserted id)
        oRoot = DBContentNode(name=sRootPath, fullpath=sRootPath)
        self.session.add(oRoot)
        self.session.commit()

        # construct a new tape
        oTape = DBContentTape(name=sTapeName, rootnode=oRoot,
                        custom_a=custom_a,
                        custom_b=custom_b,
                        custom_c=custom_c,
                        custom_d=custom_d)
        self.session.add(oTape)
        self.session.commit()

        checksums = {}
        if checksumFile:
            for l in open(checksumFile).read().splitlines():
                if l:
                    checksums['/' + l.split(' ', 2)[2]] = l.split(' ', 2)[0]

        # traverse everything below the root node
        self.lic_TraverseDirectory(oRoot, 0, checksums)

        if self.options["bAddAll"]:
            self.session.add_all(self.lNodes)
            self.session.commit()

        if self.options["iLogLevel"] > 0:
            # lets do some profiling
            t2 = time.time()
            self.log('%s took %0.3f ms' % ("traversing", (t2 - t1) * 1000.0), 1)

    def lic_getTape(self, oNode=None, sName=None):
        '''
        just get the corresponding Tape for any node in the Database (or by name)
        '''
        if sName:
            lTapes = self.session.query(DBContentTape).filter(DBContentTape.name == sName).all()
            if lTapes:
                return lTapes[0]
            else:
                return None

        oRoot = oNode.mp.query_ancestors(and_self=True).all()[0]
        try:
            oTape = self.session.query(DBContentTape).filter(
                                DBContentTape.rootnode_id == oRoot.id).one()
            return (oTape, oRoot)
        except:
            self.log("no tape found for Node %s" % oNode)
            return False

    def lic_deleteTape(self, sTapename):
        '''
        just get rid of everything in the database
        regarding this tape
        [this means the entry in "tape" table as well as
        all corresponding entries in "node" table]
        '''
        lRootNodes = []
        lTapes = self.session.query(DBContentTape).filter(
                            DBContentTape.name == sTapename).all()
        for oTape in lTapes:
            lRootNodes.extend(self.session.query(DBContentNode).filter(
                            DBContentNode.id == oTape.rootnode_id).all())
        for oTape in lTapes:
            self.session.delete(oTape)
            self.session.commit()
            print "deleting tape %s" % oTape
        for oRootNode in lRootNodes:
            self.session.delete(oRootNode)
            self.session.commit()

    def lic_FilterSearchTerm(self, sSearchTerm, bSearchContent=True, limit=10000, offset=0):
        '''
        filter the stuff in the datase according to specified searchterm
        go over all tapes, get the corresponding rootnode () and query all
        '''

        class SearchResult (object):
            pass

        class FoundNode (object):
            pass

        searchResult = SearchResult()
        searchResult.tapes = []
        searchResult.nodes = []

        if self.options["bRespectUnicode"]:
            sSearchTerm = self.string_UnicodeConvert(sSearchTerm)

        if self.options["iLogLevel"] > 0:
            t1 = time.time()

        # lookup tapes by custom fields 
        lFound = self.session.query(DBContentTape) \
                    .filter(sqlalchemy.or_(
                        DBContentTape.name.ilike('%' + sSearchTerm + '%'),
                        DBContentTape.custom_a.ilike('%' + sSearchTerm + '%'),
                        DBContentTape.custom_b.ilike('%' + sSearchTerm + '%'),
                        DBContentTape.custom_c.ilike('%' + sSearchTerm + '%'),
                        DBContentTape.custom_d.ilike('%' + sSearchTerm + '%') 
                    )) \
                    .all()
        for oTape in lFound:
            searchResult.tapes.append(oTape)

        if bSearchContent:
            # filtering the node names or their treedump beneath
            a, b = sSearchTerm, None
            op = None
            if 'AND' in sSearchTerm:
                op = sqlalchemy.and_
                a, b = sSearchTerm.split('AND', 1)
            if 'OR' in sSearchTerm:
                op = sqlalchemy.or_
                a, b = sSearchTerm.split('OR', 1)

            a = a.strip()
            if b:
                b = b.strip()

            if a.startswith('-'):
                qa = sqlalchemy.not_(DBContentNode.name.ilike('%' + a[1:] + '%'))
            else:
                qa = DBContentNode.name.ilike('%' + a + '%')

            if b:
                if b.startswith('-'):
                    qb = sqlalchemy.not_(DBContentNode.name.ilike('%' + b[1:] + '%'))
                else:
                    qb = DBContentNode.name.ilike('%' + b + '%')

            if op:
                query = op(qa, qb)
            else:
                query = qa

            query = sqlalchemy.or_(DBContentNode.checksum.ilike('%' + a + '%'), query)

            lFound = self.session.query(DBContentNode)
            lFound = lFound.filter(query)
            lFound = lFound.order_by(DBContentNode.fullpath).all()

            lFoundDump = self.session.query(DBContentNode).filter(
                        DBContentNode.treedump.ilike('%' + sSearchTerm + '%'))\
                        .all()
                        #.limit(limit).offset(offset).all()
        else:
            lFound = lFoundDump = []

        if len(lFound) > 0:
            s = "found '%s' %s time(s) (as direct Node in the database):" \
                % (sSearchTerm, len(lFound))
            self.log(s, 1)
        for oFound in lFound:
            r = self.lic_getTape(oFound)
            if not r:
                continue
            (oTape, oRoot) = r
            #s = "   [tape %s] %s" % (oTape.name, oFound.fullpath)
            #self.log(s, 1)
            searchResult.tapes.append(oTape)
            
            fn = FoundNode()
            fn.tape = oTape
            fn.path = oFound.fullpath
            fn.checksum = oFound.checksum
            
            searchResult.tapes.append(oTape)
            searchResult.nodes.append(fn)

        if len(lFoundDump) > 0:
            s = "found '%s' in treedumps of %s Nodes in the database:" \
                % (sSearchTerm, len(lFoundDump))
            self.log(s, 1)
        for oFound in lFoundDump:
            ''' paranoia check, not needed really...
            if oFound.treedump is None:
                print "hm, no treedump on %s" % oFound
                continue
            '''
            r = self.lic_getTape(oFound)
            if not r:
                continue
            (oTape, oRoot) = r
            searchResult.tapes.append(oTape)

            if self.options["bFindExactly"]:
                lLines = oFound.treedump.split('\n')[1:]
                lLinesMatch = [s for s in lLines if s.lower().find(sSearchTerm.lower()) != -1]

                # check which style the dump used for its hierarchy tokens
                # ("unicode" or "ascii", "unknown" if something went wrong)
                sDumpStyle = self.lic_CheckTreeDumpStyle(
                                        lLines[:max(len(lLines), 10)])

                s = "   [tape %s] %s time(s) in treedump beneath Node '%s':" \
                    % (oTape.name, len(lLinesMatch), oFound.fullpath)
                self.log(s, 1)
                
                for sLine in lLinesMatch:
                    iLineIndex = lLines.index(sLine)

                    if self.options["bTreeDumpFullPaths"]:
                        t = self.lic_SplitTreeLine(sLine, sDumpStyle)
                        if t:
                            (sHierarchy, sFullFile, iLevels) = t
                            fn = FoundNode()
                            fn.tape = oTape
                            fn.path = sFullFile
                            searchResult.tapes.append(oTape)
                            searchResult.nodes.append(fn)
                        else:
                            s = "      hm, someting wrong with %s ?" % sLine
                            self.log(s)

                    else:
                        # OK, this can take a loong time
                        # (not adviced to actually use this option
                        # but leaving in as reference)
                        # TODO: speed this up!
                        t = self.lic_SplitTreeLine(sLine, sDumpStyle)
                        if t:
                            (sHierarchy, sFile, iLevels) = t
                            sResult = self.lic_ConstructFullPathTree(iLineIndex,
                                                                iLevels,
                                                                lLines,
                                                                sFile,
                                                                sDumpStyle)
                            # have to append the fullpath of the root
                            # to what we found in the dump
                            sResult = "%s/%s" % (oRoot.fullpath, sResult)
                            fn = FoundNode()
                            fn.tape = oTape
                            fn.path = sResult
                            searchResult.tapes.append(oTape)
                            searchResult.nodes.append(fn)
                        else:
                            if self.options["iMaxDepth_sqlamp"] != 1:
                                s = "reached the top row %s" % sLine
                                self.log(s, 3)
            else:
                if oFound.fullpath is not None:
                    s = "  ? time(s) in treedump beneath Node '%s':" \
                        % oFound.fullpath
                    self.log(s, 1)
                    s = "    option 'bFindExactly' disabled, \
                        so being quiet here (and not waste time on finding \
                        the exact locations)..."
                    self.log(s, 1)
                else:
                    s = "hm, no fullpath on %s" % oFound
                    self.log(s, 1)

        s = "=" * 45
        self.log(s, 1)

        if self.options["iLogLevel"] > 0:
            t2 = time.time()
            s = '%s took %0.3f ms' % ("search", (t2 - t1) * 1000.0)
            self.log(s, 1)

        # make the list of tapes distinct
        searchResult.tapes = list(set(searchResult.tapes))
        searchResult.total = len(searchResult.nodes)

        newNodes = []
        nodes = sorted(searchResult.nodes, key=lambda x: x.path)

        def step(n, depth):
            nodes.remove(n)
            n.depth = depth
            newNodes.append(n)
            for node2 in nodes:
                if node2.path.startswith(n.path):
                    step(node2, depth + 1)

        for node in nodes:
            step(node, 0)

        searchResult.nodes = newNodes
        searchResult.nodes = searchResult.nodes[offset:offset+limit]

        # Chop off mountpoint
        for node in searchResult.nodes:
            if node.path.startswith('/'):
                s = node.path.split('/', 3)
                if len(s) > 3:
                    node.path = '/' + s[3]
                node.filepath = node.path
                node.filepath2 = node.path
                node.filepath3 = node.path

        return searchResult



import os

from ajenti.api import *
from ajenti.ui.binder import Binder
from ajenti.plugins.main.api import SectionPlugin
from ajenti.ui import on

from reconfigure.configs import NetatalkConfig
from reconfigure.items.netatalk import ShareData


@plugin
class Netatalk (SectionPlugin):
    config_path = '/etc/afp.conf'

    def init(self):
        self.title = 'Netatalk'
        self.icon = 'folder-close'
        self.category = _('Software')
        self.append(self.ui.inflate('netatalk:main'))

        if not os.path.exists(self.config_path):
            open(self.config_path, 'w').write("[Global]")

        self.binder = Binder(None, self.find('config'))
        self.find('shares').new_item = lambda c: ShareData()
        self.config = NetatalkConfig(path=self.config_path)

    def on_page_load(self):
        self.refresh()

    def refresh(self):
        self.config.load()
        self.binder.reset(self.config.tree).autodiscover().populate()

    @on('save', 'click')
    def on_save(self):
        self.binder.update()
        self.config.save()
        self.refresh()



from passlib.hash import sha512_crypt

import ajenti
from ajenti.api import *

from base import UserSyncProvider


@plugin
class AjentiSyncProvider (UserSyncProvider):
    id = ''
    title = _('Off')
    
    def test(self):
        return True

    def check_password(self, username, password):
        if not username in ajenti.config.tree.users:
            return False
        type = 'plain'
        saved = ajenti.config.tree.users[username].password
        if '|' in saved:
            type, saved = saved.split('|')

        if type == 'plain':
            hash = password
            return hash == saved
        elif sha512_crypt.identify(saved):
            return sha512_crypt.verify(password, saved)

    def sync(self):
        pass


import os
import subprocess


class LTFSDrive (object):
    def __init__(self):
        self.drive = ''
        self.status = ''
        self.barcode = ''
        self.origin = ''


class LTFSLibraryBackend (object):
    def __init__(self):
        self.root = '/var/lib/syslink/archive/library'

    def reload(self):
        self.drives = []
        self.mailbox = []
        self.slots = []

        for l in open(self.root + '/drives').read().splitlines():
            d = LTFSDrive()
            tokens = [_.strip() for _ in l.split(':')]
            d.drive = tokens[0].split()[3]
            d.status = tokens[1].split()[0]
            if d.status != 'Empty':
                d.origin = tokens[1].split()[3]
                if len(tokens) > 2:
                    d.barcode = tokens[2].split()[2]
            self.drives.append(d)

        for l in open(self.root + '/slots').read().splitlines():
            d = LTFSDrive()
            tokens = [_.strip() for _ in l.split(':')]
            d.drive = tokens[0].split()[2]
            d.status = tokens[1]
            if d.status != 'Empty':
                d.barcode = tokens[2].split('=')[1]
            self.slots.append(d)

        for l in open(self.root + '/mailbox').read().splitlines():
            d = LTFSDrive()
            tokens = [_.strip() for _ in l.split(':')]
            d.drive = tokens[0].split()[2]
            d.status = tokens[1]
            if d.status != 'Empty':
                d.barcode = tokens[2].split('=')[1]
            self.mailbox.append(d)

    def run_action(self, action, *args):
        subprocess.Popen([os.path.join(self.root, '../' + action + '.sh')] + list(args))



from ajenti.api import *
from ajenti.plugins.main.api import SectionPlugin
from ajenti.ui import on
from ajenti.ui.binder import Binder

from backend import *


@plugin
class LTFSLibraryPlugin (SectionPlugin):
    def init(self):
        self.title = 'Library'
        self.icon = 'reorder'
        self.category = 'LTFS'

        self.append(self.ui.inflate('ltfs:library-main'))
        self.backend = LTFSLibraryBackend()

        self.binder = Binder(self.backend, self)

        def post_drive_bind(object, collection, item, ui):
            ui.find('unload').on('click', self.on_unload, item)

        self.find('drives').post_item_bind = post_drive_bind

    def refresh(self):
        self.backend.reload()
        self.binder.reset().autodiscover().populate()

        l = list(set(self.backend.slots + self.backend.mailbox))
        l = sorted(l, key=lambda x: int(x.drive))

        def make_props(l):
            labels = ['%s [%s] (%s)' % (_.drive, _.status, _.barcode) for _ in l]
            values = [_.drive for _ in l]
            return labels, values

        list_empty = filter(lambda x: x.status == 'Empty', l)
        list_full = filter(lambda x: x.status == 'Full', l)
        self.find('source-slot').labels, self.find('source-slot').values = make_props(list_full)
        self.find('dest-slot').labels, self.find('dest-slot').values = make_props(list_empty)

    def _find_slot(self, id):
        for s in self.backend.slots + self.backend.mailbox:
            if s.drive == id:
                return s

    def on_page_load(self):
        self.refresh()

    @on('refresh', 'click')
    def on_refresh(self):
        self.refresh()

    @on('inquiry', 'click')
    def on_inquiry(self):
        self.backend.run_action('libstatus')
        self.context.notify('info', 'Inquiry started')

    @on('load', 'click')
    def on_load(self):
        item = self._find_slot(self.find('source-slot').value)
        self.backend.run_action('loadtape', item.barcode)
        self.context.notify('info', 'Loading. Please wait and refresh!')
        self.refresh()

    def on_unload(self, item):
        self.backend.run_action('unloadtape', item.drive)
        self.context.notify('info', 'Unloading. Please wait and refresh!')
        self.refresh()

    @on('move', 'click')
    def on_move(self):
        item = self._find_slot(self.find('source-slot').value)
        ditem = self._find_slot(self.find('dest-slot').value)
        self.backend.run_action('movetape', item.drive, ditem.drive)
        self.context.notify('info', 'Moving. Please wait and refresh!')
        self.refresh()

    @on('format', 'click')
    def on_format(self):
        item = self._find_slot(self.find('source-slot').value)
        self.backend.run_action('formattape', item.barcode)
        self.context.notify('info', 'Formatting. Please wait and refresh!')
        self.refresh()

    @on('fsck', 'click')
    def on_fsck(self):
        item = self._find_slot(self.find('source-slot').value)
        self.backend.run_action('fscktape', item.barcode)
        self.context.notify('info', 'Checking. Please wait and refresh!')
        self.refresh()



from ajenti.api import *
from ajenti.plugins import *


info = PluginInfo(
    title='Tasks',
    icon='cog',
    dependencies=[
        PluginDependency('main'),
        PluginDependency('cron'),
    ],
)


def init():
    import api
    import manager
    import main
    import tasks



from ajenti.api import *
from ajenti.plugins import *


info = PluginInfo(
    title='PostgreSQL',
    icon='table',
    dependencies=[
        PluginDependency('db_common'),
        BinaryDependency('psql'),
    ],
)


def init():
    import main



from ajenti.api import *
from ajenti.plugins.main.api import SectionPlugin
from ajenti.ui import on
from ajenti.ui.binder import Binder

from reconfigure.configs import HostsConfig
from reconfigure.items.hosts import AliasData, HostData


@plugin
class Hosts (SectionPlugin):
    def init(self):
        self.title = _('Hosts')
        self.icon = 'sitemap'
        self.category = _('System')

        self.append(self.ui.inflate('hosts:main'))

        self.config = HostsConfig(path='/etc/hosts')
        self.binder = Binder(None, self.find('hosts-config'))
        self.find('aliases').new_item = lambda c: AliasData()
        self.find('hosts').new_item = lambda c: HostData()

    def on_page_load(self):
        self.config.load()
        self.binder.reset(self.config.tree).autodiscover().populate()

    @on('save', 'click')
    def save(self):
        self.binder.update()
        self.config.save()



from ajenti.api import *
from ajenti.ui import p, UIElement


@p('width', default=None)
@p('height', default=None)
@p('scroll', default=False, type=bool)
@plugin
class Box (UIElement):
    typeid = 'box'


@p('text', default='', bindtypes=[str, unicode])
@plugin
class FormLine (UIElement):
    typeid = 'formline'


@p('text', default='', bindtypes=[str, unicode])
@plugin
class FormGroup (UIElement):
    typeid = 'formgroup'


@p('expanded', default=False, type=bool, bindtypes=[bool])
@plugin
class Collapse (UIElement):
    typeid = 'collapse'


@p('expanded', default=False, type=bool, bindtypes=[bool])
@plugin
class CollapseRow (UIElement):
    typeid = 'collapserow'


@p('width', default='99%')
@plugin
class Table (UIElement):
    typeid = 'dt'


@p('sortable', default=True, type=bool)
@p('order', default='', type=str)
@plugin
class SortableTable (Table):
    typeid = 'sortabledt'


@p('width', default=None)
@p('forcewidth', default=None)
@plugin
class TableCell (UIElement):
    typeid = 'dtd'


@p('text', default='', bindtypes=[str, unicode])
@p('width', default=None)
@plugin
class TableHeader (UIElement):
    typeid = 'dth'


@p('width', default='99%')
@plugin
class LayoutTable (UIElement):
    typeid = 'lt'


@p('width', default=None)
@plugin
class LayoutTableCell (UIElement):
    typeid = 'ltd'


@p('title', default='', bindtypes=[str, unicode])
@plugin
class Tab (UIElement):
    typeid = 'tab'


@p('active', default=0)
@plugin
class Tabs (UIElement):
    typeid = 'tabs'

    def on_switch(self):
        self.children_changed = True  # force update



from ajenti.api import *
from ajenti.plugins import manager

import binder
from inflater import Inflater
from element import p, on, UIElement


@plugin
class UI (BasePlugin):
    """
    The root UI object, one per session
    """

    def init(self):
        self.inflater = Inflater.get(manager.context)

    def create(self, *args, **kwargs):
        """
        Creates an element by its type ID.
        """
        return self.inflater.create_element(self, *args, **kwargs)

    def inflate(self, layout):
        """
        :returns: an inflated element tree of the given layout XML name
        """
        return self.inflater.inflate(self, layout)

    def render(self):
        """
        Renders the UI into JSON
        """
        return self.root.render()

    def find(self, id):
        """
        :returns: nearest element with given ID
        """
        return self.root.find(id)

    def find_uid(self, uid):
        """
        :returns: nearest element with given unique ID
        """
        return self.root.find_uid(uid)

    def dispatch_event(self, uid, event, params=None):
        """
        Dispatches an event to an element with given UID
        """
        self.root.dispatch_event(uid, event, params)

    def has_updates(self):
        """
        Checks for pending UI updates
        """
        return self.root.has_updates()

    def clear_updates(self):
        """
        Marks all pending updates as processed
        """
        return self.root.clear_updates()


__all__ = ['UI', 'UIElement', 'p', 'on', 'binder']



import gevent

from ajenti.api import *
from ajenti.plugins.main.api import SectionPlugin
from ajenti.ui import on
from ajenti.ui.binder import Binder

from backend import *
from ..tapegroups.backend import *


@plugin
class LTFSJobsPlugin (SectionPlugin):
    def init(self):
        self.title = 'Jobs'
        self.icon = 'hdd'
        self.category = 'LTFS'

        self.append(self.ui.inflate('ltfs:jobs-main'))
        self.backend = LTFSJobsBackend()
        self.tgbackend = TapeGroupBackend()

        self.binder = Binder(self.backend, self)
        self.add_binder = Binder(None, self.find('add-dialog'))

        self.adding_job = None

        def post_job_bind(object, collection, item, ui):
            ui.find('resume').visible = item.status.upper() == 'PAUSED'
            ui.find('pause').visible = item.status.upper() != 'PAUSED'
            ui.find('cancel').visible = item.status.upper() in ['FINISHED', 'PAUSED', 'ERROR']
            ui.find('log').on('click', self.on_view_log, item)
            ui.find('files').on('click', self.on_list_files, item)
            ui.find('pause').on('click', self.on_action, 'pause', item)
            ui.find('resume').on('click', self.on_action, 'resume', item)
            ui.find('cancel').on('click', self.on_action, 'cancel', item)

        self.find('jobs').post_item_bind = post_job_bind

    def refresher(self):
        while True:
            self.refresh()
            gevent.sleep(120)

    def refresh(self):
        self.backend.reload()
        self.tgbackend.reload()
        self.binder.reset().autodiscover().populate()
        if self.adding_job:
            self.add_binder.reset(self.adding_job).autodiscover().populate()

    def on_page_load(self):
        self.context.endpoint.spawn(self.refresher)
        self.refresh()

    @on('refresh', 'click')
    def on_refresh(self):
        self.refresh()

    def on_list_files(self, job):
        self.find('file-list-dialog').visible = True
        self.find('file-list-dialog').find('list').value = '\n'.join(f.path for f in job.source)

    def on_view_log(self, job):
        self.context.launch('view-log', path=self.backend.get_log_path(job))

    @on('file-list-dialog', 'button')
    def on_file_list_dialog_close(self, button=None):
        self.find('file-list-dialog').visible = False

    def on_action(self, action, job):
        self.backend.run_action(action, job)
        self.refresh()

    @on('add-job', 'click')
    def on_add(self):
        self.adding_job = LTFSJob()
        self.find('add-dialog').visible = True

        groups = [None] + [g.id for g in self.tgbackend.groups]
        groupl = ['None'] + [g.name for g in self.tgbackend.groups]

        self.find('add-dialog').find('group').values = groups
        self.find('add-dialog').find('group').labels = groupl
        self.find('add-dialog').find('group2').values = groups
        self.find('add-dialog').find('group2').labels = groupl

        self.refresh()

    @on('add-source-file', 'click')
    def on_add_file(self):
        self.add_binder.update()
        self.find('add-file-dialog').visible = True

    @on('add-source-dir', 'click')
    def on_add_dir(self):
        self.add_binder.update()
        self.find('add-dir-dialog').visible = True

    @on('add-dialog', 'button')
    def on_dialog(self, button=None):

        self.add_binder.update()
        if button == 'save':
            if len(self.adding_job.source) == 0:
                self.context.notify('error', 'Please add at least one source')
            j1 = self.adding_job
            j2 = self.adding_job.clone()
            j2.destination = j1.destination2
            j2.group = j1.group2
            j2.export = j1.export2

            if j1.group != 'null':
                self.backend.create_job(j1)

            if j2.group != 'null':
                self.backend.create_job(j2)

            if j1.group != 'null' and j2.group != 'null':
                j2.has_copy2 = 'yes'
                j2.copy2_id = j1.id
                self.backend.resave_job(j2)

                j1.has_copy2 = 'yes'
                j1.copy2_id = j2.id
                self.backend.resave_job(j1)

            self.refresh()

        self.find('add-dialog').visible = False
        self.adding_job = None

    @on('add-file-dialog', 'select')
    def on_add_file_dialog_select(self, path=None):
        self.find('add-file-dialog').visible = False
        if path:
            self.adding_job.source.append(LTFSSource(path))
            self.refresh()

    @on('add-dir-dialog', 'select')
    def on_add_dir_dialog_select(self, path=None):
        self.find('add-dir-dialog').visible = False
        if path:
            self.adding_job.source.append(LTFSSource(path))
            self.refresh()



import os
import subprocess


class LTFSSource (object):
    def __init__(self, path):
        self.path = path


class LTFSJob (object):
    def __init__(self):
        self.id = 0
        self.source = []

        self.group = '0'
        self.selected_tape = 'UNASSIGNED'
        self.has_copy2 = 'no'
        self.copy2_id = ''
        self.status = 'QUEUED'
        self.copied = '0'
        self.total = '0'
        self.type = 'BACKUP'
        self.export = False
        self.start = '--:--'
        self.duration = '--:--'
        self.name = ''
        self.destination = ''
        self.delete_source = False

        self.destination2 = ''
        self.group2 = None
        self.export2 = False

    def clone(self):
        j = LTFSJob()
        j.__dict__.update(self.__dict__)
        return j


class LTFSJobsBackend (object):
    def __init__(self):
        self.jobs = []
        self.root = '/var/lib/syslink/archive'

    def parse_job(self, id, file):
        lines = file.read().splitlines()
        j = LTFSJob()
        j.id = id

        (j.group, j.selected_tape, j.has_copy2, j.copy2_id,
         j.status,  j.copied, j.total, j.type, j.export,
         j.start, j.duration, j.name, j.destination,
         j.delete_source) = lines[:14]

        j.export = j.export == 'yes'
        j.delete_source = j.delete_source == 'yes'
        j.progress = 1.0 * int(j.copied) / (int(j.total) + 1)
        return j

    def save_job(self, j, file):
        file.write('\n'.join((
            j.group, j.selected_tape, j.has_copy2, str(j.copy2_id),
            j.status,  j.copied, j.total, j.type, 'yes' if j.export else 'no',
            j.start, j.duration, j.name, j.destination,
            'yes' if j.delete_source else 'no',
        )) + '\n')

    def save_job_files(self, job, file):
        file.write('\n'.join(_.path for _ in job.source) + '\n')

    def get_log_path(self, job):
        return self.get_path(job, dir='logs') + '.log'

    def get_path(self, job, dir='jobs'):
        return os.path.join(self.root, dir, str(job.id))

    def run_action(self, action, job):
        subprocess.call([os.path.join(self.root, action + '.sh'), str(job.id)])
        self.reload()

    def create_job(self, job):
        id = 0
        for f in os.listdir(os.path.join(self.root, 'jobs')):
            if not '.' in f:
                if int(f) >= id:
                    id = int(f) + 1

        job.id = id
        self.save_job(job, open(self.get_path(job), 'w'))
        self.save_job_files(job, open(self.get_path(job) + '.files', 'w'))
        os.symlink(self.get_path(job), self.get_path(job, 'queue'))
        self.reload()

    def resave_job(self, job):
        self.save_job(job, open(self.get_path(job), 'w'))

    def reload(self):
        self.jobs = []

        for f in os.listdir(os.path.join(self.root, 'jobs')):
            if not '.' in f:
                j = self.parse_job(int(f), open(os.path.join(self.root, 'jobs', f)))
                j.source = [
                    LTFSSource(x)
                    for x in filter(None, open(os.path.join(self.root, 'jobs', f + '.files')).read().splitlines())
                ]
                self.jobs.append(j)

        self.jobs = sorted(self.jobs, key=lambda x: -x.id)



# coding=utf-8
"""
OpenVPN Management API for Python
(http://svn.openvpn.net/projects/openvpn/contrib/dazo/management/management-notes.txt)
"""
__author__ = "Ilya Voronin"
__version__ = "0.1"
__all__ = ['manager']

import socket

CRLF = "\r\n"
PASSWORD_PROMPT = "ENTER PASSWORD:"
STATUS_SUCCESS = "SUCCESS: "
STATUS_ERROR = "ERROR: "
END_MARKER = "END"
NOTIFICATION_PREFIX = ">"


class lsocket(socket.socket):
    """Socket subclass for buffered line-oriented i/o
    @version: 1.0
    @group Line functions: recvl, peekl, sendl, recvuntil
    @group Byte functions: recvb, peekb
    @sort: sendl,recvl,peekl,recvuntil,recvb,peekb
    @ivar _b: Internal read buffer"""
    def __init__(self, *args, **kwargs):
        """Calls superclass constructor and initializes instance variables."""
        super(lsocket, self).__init__(*args, **kwargs)
        self._b = ""

    def sendl(self, line):
        """Adds CRLF to the supplied line and sends it
        @type line: str
        @param line: Line to send"""
        #print "> {0}".format(line)
        self.send(line + CRLF)

    def recvl(self, peek=False):
        """Fills internal buffer with data from socket and returns first found line.
        @type peek: Bool
        @param peek: Keeps line in the buffer if true
        @rtype: str
        @return: First found line"""
        while True:
            i = self._b.find(CRLF)
            if i == -1:
                self._b += self.recv(1024)
            else:
                line = self._b[:i]
                if not peek:
                    self._b = self._b[i+2:]
                #print "{0} {1}".format("?" if peek else ">", line)
                return line

    def recvb(self, count, peek=False):
        """Fills internal buffer with data from socket and returns first B{count} received bytes.
        @type count: int
        @param count: Number of bytes to recieve
        @type peek: Bool
        @param peek: Keeps bytes in the buffer if true
        @rtype: str
        @return: First B{count} received bytes"""
        while True:
            if count > len(self._b):
                self._b += self.recv(count - len(self._b))
            else:
                bytez = self._b[:count]
                if not peek:
                    self._b = self._b[count:]
                #print "{0} {1}".format("?" if peek else ">", bytez)
                return bytez

    def peekb(self, count):
        """Same as peekb(count, peek = True)
        @type count: int
        @param count: Number of bytes to peek
        @rtype: str
        @return: First B{count} received bytes"""
        return self.recvb(count, peek=True)

    def peekl(self):
        """Same as recvl(peek = True)
        @rtype: str
        @return: First found line"""
        return self.recvl(peek=True)

    def recvuntil(self, marker):
        """Returns list of lines received until the line matching the B{marker} was found.
        @type marker: str
        @param marker: Marker
        @rtype: list
        @return: List of strings"""
        lines = []
        while True:
            line = self.recvl()
            if line == marker:
                return lines
            lines.append(line)


class manager(object):
    """OpenVPN Management Client.
    @group Connection: connect,disconnect
    @group Commands: status, stats, killbyaddr, killbycn, signal, messages, pid
    @ivar _host: Hostname or address to connect to
    @ivar _port: Port number to connect to
    @ivar _path: Path to UNIX socket to connect to
    @ivar _password: Password
    @ivar _timeout: Connection timeout
    @ivar _s: Socket object"""
    def __init__(self, host=None, port=None, path=None, password="", timeout=5):
        """Initializes instance variables.
        @type host: str
        @param host: Hostname or address to connect to
        @type port: int
        @param port: Port number to connect to
        @type path: str
        @param path: Path to UNIX socket to connect to
        @type password: str
        @param password: Password
        @type timeout: int
        @param timeout: Connection timeout
        @note: host/port and path should not be specified at the same time """
        assert bool(host and port) ^ bool(path)
        (self._host, self._port, self._path, self._password, self._timeout) = (host, port, path, password, timeout)
        self._s = None

    def connect(self):
        """Initializes and opens a socket. Performs authentication if needed."""
        # UNIX Socket
        if self._path:
            self._s = lsocket(socket.AF_UNIX)
            self._s.settimeout(self._timeout)
            self._s.connect(self._path)
        # IPv4 or IPv6 Socket
        else:
            ai = socket.getaddrinfo(self._host, self._port, 0, socket.SOCK_STREAM)[0]
            self._s = lsocket(ai[0], ai[1], ai[2])
            self._s.settimeout(self._timeout)
            self._s.connect((self._host, self._port))

        # Check if management is password protected
        # We cannot use self.execute(), because OpenVPN uses non-CRLF terminated lines during auth
        if self._s.peekb(len(PASSWORD_PROMPT)) == PASSWORD_PROMPT:
            self._s.recvb(len(PASSWORD_PROMPT))
            self._s.sendl(self._password)
            # Check if OpenVPN asks for password again
            if self._s.peekb(len(PASSWORD_PROMPT)) == PASSWORD_PROMPT:
                self._s.recvb(len(PASSWORD_PROMPT)) # Consume "ENTER PASSWORD:"
                raise Exception("Authentication error")
            else:
                self._s.recvl() # Consume "SUCCESS: " line

    def disconnect(self):
        """Closes a socket."""
        self._s.close()
        self._s = None

    @property
    def connected(self):
        """Returns socket status."""
        return bool(self._s)

    def execute(self, name, *args):
        """Executes command and returns response as a list of lines.
        @type name: str
        @param name: Command name
        @type args: list
        @param args: Argument List
        @rtype: list
        @return: List of strings"""
        cmd = name
        if args:
            cmd += " " + " ".join(args)
        self._s.sendl(cmd)

        # Handle notifications
        while self._s.peekl().startswith(NOTIFICATION_PREFIX):
            self._s.recvl()  # Consume

        # Handle one-line responses
        if self._s.peekl().startswith(STATUS_SUCCESS):
            return [self._s.recvl()[len(STATUS_SUCCESS):]]
        elif self._s.peekl().startswith(STATUS_ERROR):
            raise Exception(self._s.recvl()[len(STATUS_ERROR):])

        # Handle multi-line reponses
        return self._s.recvuntil(END_MARKER)

    def status(self):
        """Executes "status 2" command and returns response as a dictionary.
        @rtype: dict
        @return: Dictionary"""
        status = dict()
        status["clients"] = list()
        # V2 status
        for line in self.execute("status", "2"):
            fields = line.split(",")
            if fields[0] == "TITLE":
                status["title"] = fields[1]
            elif fields[0] == "TIME":
                status["time"] = fields[1]
                status["timeu"] = fields[2]
            elif fields[0] == "HEADER":
                continue
            elif fields[0] == "CLIENT_LIST":
                status["clients"].append(dict(
                    zip(("cn", "raddress", "vaddress", "brecv", "bsent", "connsince", "connsinceu"), fields[1:])
                ))

        return status

    def messages(self):
        """Executes "log all" and returns response as a list of messages.
        Each message is a list of 3 elements: time, flags and text.
        @rtype: list
        @return: List of messages"""
        return map(lambda x: x.split(","), self.execute("log", "all"))

    def killbyaddr(self, addr):
        """Executes "kill IP:port" command and returns response as a string
        @type addr: str
        @param addr: Real address of client to kill (in IP:port format)
        @rtype: str
        @return: Response string"""
        return self.execute("kill", addr)[0]

    def killbycn(self, cn):
        """
        Executes "kill cn" command and returns response as a string
        @type cn: str
        @param cn: Common name of client(s) to kill
        @rtype: str
        @return: Response string
        """
        return self.execute("kill", cn)[0]

    @staticmethod
    def _pgenresp(response):
        """
        Parses generically formatted response (param1=value1,param2=value2,param3=value3)
        @type response: str
        @param response: Response string
        @return: Dictionary
        """
        return dict(map(lambda x: x.split("="), response.split(",")))

    def stats(self):
        """Executes "load-stats" command and returns response as a dictionary:

        >>> print manager.stats()
        {'nclients': '2', 'bytesout': '21400734', 'bytesin': '10129283'}

        @rtype: dict
        @return: Dictionary"""
        return self._pgenresp(self.execute("load-stats")[0])

    def signal(self, signal):
        """Executes "signal" command and returns response as a string.
        @type signal: str
        @param signal: Signal name (SIGHUP, SIGTERM, SIGUSR1 or SIGUSR2)
        @rtype: str
        @return: Response string"""
        assert signal in ["SIGHUP", "SIGTERM", "SIGUSR1", "SIGUSR2"]
        return self.execute("signal", signal)[0]

    def pid(self):
        """Executes "pid" command and returns response as a dictionary.
        @return: Dictionary"""
        return self._pgenresp(self.execute("pid")[0])["pid"]



from ajenti.api import *
from ajenti.plugins import *


info = PluginInfo(
    title='LTFS Archive',
    icon='list',
    dependencies=[
        PluginDependency('main_'),
    ],
)


def init():
    import archive.main
    import jobs.main
    import library.main
    import tapegroups.main
